# This source file is part of Yoda.
#
# Yoda is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# Yoda is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License along with
# Yoda. If not, see <http://www.gnu.org/licenses/gpl-3.0.html>.

import logging
import os
import shutil

from os import listdir
from os.path import join
from prettytable import PrettyTable
from pycolorizer import Color

from yoda import Repository
from yoda.repository import clone
from yoda import RepositoryError
from yoda.subcommands import Subcommand
from yoda import Workspace as Ws
from yoda import yn_choice


class Workspace(Subcommand, object):
    ws = None
    subparser = None
    logger = None

    def setup(self, name, config, subparser):
        self.ws = Ws(config)
        self.subparser = subparser
        self.logger = logging.getLogger(__name__)
        super(Workspace, self).setup(name, config, subparser)

    def parse(self):
        parser = self.subparser.add_parser(
            "workspace",
            help="Workspace managment",
            description="Manage repository's workspace")
        subparser = parser.add_subparsers(
            dest="workspace_subcommand")

        add_parser = subparser.add_parser(
            "add", help="Add workspace")
        rm_parser = subparser.add_parser(
            "remove", help="Remove existing workspace")
        subparser.add_parser(
            "list", help="Show registered workspace")

        add_parser.add_argument("name", type=str, help="Workspace name")
        add_parser.add_argument("path", type=str, help="Workspace path")
        rm_parser.add_argument("name", type=str, help="Workspace name")

    def execute(self, args):
        if args.workspace_subcommand is None:
            self.parser.print_help()
            return None

        color = Color()
        if (args.workspace_subcommand == "add"):
            self.ws.add(args.name, args.path)
            self.logger.info(color.colored(
                "Workspace `%s` successfuly added" % args.name, "green"))
        elif (args.workspace_subcommand == "remove"):
            self.ws.remove(args.name)
            self.logger.info(color.colored(
                "Workspace `%s` successfuly removed" % args.name, "green"))
        elif (args.workspace_subcommand == "list"):
            table = PrettyTable(["Name", "Path"])
            table.align["Name"] = "l"
            table.align["Path"] = "l"
            for key, ws in sorted(self.ws.list().items()):
                table.add_row([key, ws["path"]])

            self.logger.info(table)

    def load_workspaces_subcommands(self, subcmd):
        for key, value in self.ws.list().items():
            ws_subcmds = WorkspaceSubcommands(key, self.subparser, self.config)
            subcmd.commands[key] = ws_subcmds


class WorkspaceSubcommands():
    name = None
    parser = None
    config = None
    logger = None

    def __init__(self, name, subparser, config):
        """Initialize workspace name."""
        self.name = name
        self.parser = subparser.add_parser(
            name,
            description="Manage repositories in %s workspace" % name)
        self.config = config
        self.logger = logging.getLogger(__name__)

    def parse(self):
        subparser = self.parser.add_subparsers(dest="action")

        add_parser = subparser.add_parser(
            "add", help=("Add repository to %s workspace" % self.name))

        add_parser.add_argument("repo_name", type=str, help="Repository name")
        add_parser.add_argument(
            "-u", "--url",
            type=str, help="Repository url",
            default=None, nargs='?'
        )
        add_parser.add_argument(
            "-p", "--path",
            type=str, help="Repository path",
            default=None, nargs='?'
        )

        remove_parser = subparser.add_parser(
            "remove", help=("Remove repository from %s workspace" % self.name))
        remove_parser.add_argument(
            "repo_name", type=str, help="Repository name"
        )

        sync_parser = subparser.add_parser(
            "sync", help="Synchronize all directories store in workspace"
        )
        sync_parser.add_argument(
            "name", type=str, help="Workspace name", nargs='?'
        )

    def execute(self, args):
        if (args.action == "add"):
            self.add(args.subcommand, args.repo_name, args.url, args.path)
        elif (args.action == "remove"):
            self.remove(args.subcommand, args.repo_name)
        elif (args.action == "sync"):
            self.sync(args.subcommand)

    def add(self, ws_name, repo_name, url, path):
        ws = self.config["workspaces"][ws_name]
        repo_path = ws["path"] + "/" + repo_name if path is None else path

        if ("repositories" not in ws):
            ws["repositories"] = {}

        if (repo_name in ws["repositories"]):
            raise ValueError("Repository %s already exists" % repo_name)

        if url is not None:
            clone(url, repo_path)

        if (os.path.exists(repo_path) is False):
            os.mkdir(repo_path)

        ws["repositories"][repo_name] = repo_path
        self.logger.info("Repository %s added" % repo_name)

    def remove(self, ws_name, repo_name):
        config = self.config
        if (repo_name not in config["workspaces"][ws_name]["repositories"]):
            raise ValueError(
                "%s not found in %s workspace" % (repo_name, ws_name)
            )

        repo_path = config["workspaces"][ws_name]["repositories"][repo_name]
        del config["workspaces"][ws_name]["repositories"][repo_name]

        if (yn_choice("Do you want to delete this repository?")):
            shutil.rmtree(repo_path)

    def sync(self, ws_name):
        """Synchronise workspace's repositories."""
        path = self.config["workspaces"][ws_name]["path"]
        repositories = self.config["workspaces"][ws_name]["repositories"]

        repo_list = {}

        for r in listdir(path):
            try:
                repo = Repository(join(path, r))
            except RepositoryError:
                continue
            else:
                repositories[r] = repo.path
                repo_list[r] = repo.path

        logger = logging.getLogger(__name__)
        color = Color()
        logger.info("Workspace `%s` synchronized" % ws_name)
        logger.info("Added %d repositories:" % len(repo_list))
        for repo_name, path in repo_list.items():
            logger.info(color.colored(
                " - %s" % repo_name, "blue"))



# This source file is part of Yoda.
#
# Yoda is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# Yoda is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License along with
# Yoda. If not, see <http://www.gnu.org/licenses/gpl-3.0.html>.

from os.path import exists
from yoda import Config


class Workspace:

    config = None

    def __init__(self, config):
        """Workspace object initialization."""
        if not isinstance(config, Config):
            raise TypeError("Variable config is not an instance of Config")
        self.config = config

    def add(self, name, path):
        """Add a workspace entry in user self.config file."""
        if not (exists(path)):
            raise ValueError("Workspace path `%s` doesn't exists." % path)

        if (self.exists(name)):
            raise ValueError("Workspace `%s` already exists." % name)

        self.config["workspaces"][name] = {"path": path, "repositories": {}}

    def remove(self, name):
        """Remove workspace from self.config file."""
        if not (self.exists(name)):
            raise ValueError("Workspace `%s` doesn't exists." % name)

        self.config["workspaces"].pop(name, 0)

    def list(self):
        """List all available workspaces."""
        ws_list = {}

        for key, value in self.config["workspaces"].items():
            ws_list[key] = dict({"name": key}, **value)

        return ws_list

    def get(self, name):
        """
        Get workspace infos from name.
        Return None if workspace doesn't exists.
        """
        ws_list = self.list()
        return ws_list[name] if name in ws_list else None

    def exists(self, name):
        """Check if given workspace name exists."""
        return name in self.list().keys()

    def repository_exists(self, workspace, repo):
        """Return True if workspace contains repository name."""
        if not self.exists(workspace):
            return False

        workspaces = self.list()
        return repo in workspaces[workspace]["repositories"]



# This source file is part of Yoda.
#
# Yoda is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# Yoda is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License along with
# Yoda. If not, see <http://www.gnu.org/licenses/gpl-3.0.html>.

import os.path
import yaml


class Config(dict):

    config_file = None

    def __init__(self, config_file, *args, **kwargs):
        """Workspace object initialization."""
        self.config_file = config_file
        super(Config, self).__init__(*args, **kwargs)
        if not os.path.exists(config_file) or config_file is None:
            self.update({"workspaces": {}})
            self.write()
        else:
            file = open(self.config_file)
            config = yaml.load(file.read())
            file.close()
            if config is not None:
                self.update(config)

    def __delitem__(self, key):
        super(Config, self).__delitem__(key)
        self.write()

    def __setitem__(self, key, value):
        super(Config, self).__setitem__(key, value)
        self.write()

    def update(self, *args, **kwargs):
        super(Config, self).update(*args, **kwargs)
        self.write()

    def write(self):
        """
        Write config in configuration file.
        Data must me a dict.
        """
        file = open(self.config_file, "w+")
        file.write(yaml.dump(dict(self), default_flow_style=False))
        file.close()



# This source file is part of Yoda.
#
# Yoda is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# Yoda is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License along with
# Yoda. If not, see <http://www.gnu.org/licenses/gpl-3.0.html>.

from yoda.adapter import Abstract


class Git(Abstract):
    """Git Adapter."""

    executable = "git"

    def status(self):
        """Show git status."""
        return self.exec_on_path("git status")

    def clone(self, url):
        """Clone repository from url."""
        return self.execute("git clone %s %s" % (url, self.path))

    def update(self):
        """Update repository."""
        return self.exec_on_path("git pull --rebase")

    def show(self):
        """Show repository details."""
        raise NotImplemented("TODO: Not implemented")



# This source file is part of Yoda.
#
# Yoda is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# Yoda is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License along with
# Yoda. If not, see <http://www.gnu.org/licenses/gpl-3.0.html>.

import os

from yoda.adapter import Git


class RepositoryError(Exception):
    """Generic repository error."""
    pass


class RepositoryPathInvalid(RepositoryError):
    """Repository path is invalid, doesn't exists or is not a directory."""
    pass


class RepositoryAdapterNotFound(RepositoryError):
    """Repository invalid because adapter not found."""
    pass


class Repository:
    path = None
    adapter = None

    scm_dirs = [".git"]

    def __init__(self, path):
        if not os.path.exists(path) or not os.path.isdir(path):
            raise RepositoryPathInvalid(
                "Path doesn't exists or isn't a directory (%s)\n" % path)

        try:
            scm = (set(self.scm_dirs) & set(os.listdir(path))).pop()
        except KeyError:
            raise RepositoryAdapterNotFound("Can't define repository type")
        else:
            self.path = path
            if scm == ".git":
                self.adapter = Git(path)

    def get_scm(self):
        """Get scm used as string."""
        if self.adapter is None:
            return None
        return self.adapter.__class__.__name__

    def status(self):
        return self.adapter.status()

    def update(self):
        return self.adapter.update()


def clone(url, path):
    """Clone a repository."""
    adapter = None
    if url[:4] == "git@" or url[-4:] == ".git":
        adapter = Git(path)

    if adapter is None:
        raise RepositoryAdapterNotFound(
            "Can't find adapter for `%s` repository url" % url)

    adapter.clone(url)



# This source file is part of Yoda.
#
# Yoda is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# Yoda is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License along with
# Yoda. If not, see <http://www.gnu.org/licenses/gpl-3.0.html>.

import copy
import logging
import os
import traceback

from prettytable import PrettyTable

BLACK, RED, GREEN, YELLOW, BLUE, MAGENTA, CYAN, WHITE = range(8)
RESET_SEQ = "\033[0m"
COLOR_SEQ = "\033[1;%dm"
COLORS = {
    'WARNING': YELLOW,
    'INFO': WHITE,
    'DEBUG': BLUE,
    'CRITICAL': RED,
    'ERROR': RED
}
FORMAT = """[%(asctime)s] [%(levelname)s] <%(name)s> - \
%(message)s (%(filename)s:%(lineno)d)"""
LFORMAT = "%(levelname)s: %(message)s"


class Formatter(logging.Formatter):
    def format(self, record):
        rec = copy.copy(record)

        if isinstance(rec.msg, Exception) and self._fmt == FORMAT:
            rec.msg = "%s\n%s" % (rec.msg, traceback.format_exc())

        if isinstance(rec.msg, PrettyTable):
            rec.msg = "\n%s" % rec.msg

        level = record.levelname

        if level in COLORS:
            lvlname = COLOR_SEQ % (30 + COLORS[level]) + level + RESET_SEQ
            rec.levelname = lvlname

        if level == "INFO" and self._fmt != FORMAT:
            formatter = Formatter("%(message)s")
            return formatter.format(rec)

        return logging.Formatter.format(self, rec)


class Logger(logging.Logger):
    def __init__(self, name):
        logging.Logger.__init__(self, name, logging.DEBUG)

    def set_file_handler(self, logfile):
        """Set FileHandler"""
        handler = logging.FileHandler(logfile)
        handler.setLevel(logging.NOTSET)
        handler.setFormatter(Formatter(FORMAT))

        self.addHandler(handler)

    def set_console_handler(self, debug=False):
        """Set Console handler."""
        console = logging.StreamHandler()
        console.setFormatter(Formatter(LFORMAT))
        if not debug:
            console.setLevel(logging.INFO)

        self.addHandler(console)



# This source file is part of Yoda.
#
# Yoda is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# Yoda is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License along with
# Yoda. If not, see <http://www.gnu.org/licenses/gpl-3.0.html>.

from abc import ABCMeta
from abc import abstractmethod
from distutils.spawn import find_executable
import subprocess


class Abstract:
    """SCM Adapter interface."""
    __metaclass__ = ABCMeta

    path = None
    executable = None

    def __init__(self, path):
        self.path = path

    def execute(self, command, path=None):
        """Execute command with os.popen and return output."""
        self.check_executable()
        subprocess.Popen(command, shell=True, cwd=path).communicate()

    def exec_on_path(self, command):
        """Execute command in repository path."""
        self.execute("%s" % (command), self.path)

    def check_executable(self):
        """Check adapter executable exists."""
        if not find_executable(self.executable):
            raise Exception("Executable %s not found" % self.executable)

    @abstractmethod
    def status(self):
        pass

    @abstractmethod
    def update(self):
        pass

    @abstractmethod
    def show(self):
        pass

    @abstractmethod
    def clone(self):
        pass



# This source file is part of Yoda.
#
# Yoda is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# Yoda is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License along with
# Yoda. If not, see <http://www.gnu.org/licenses/gpl-3.0.html>.

import argparse

from yoda.version import get_version


class Subcommand:
    config = None
    parser = None

    def setup(self, name, config, subparser):
        self.parser = subparser.add_parser(name)
        self.config = config


class Subcommands:
    config = None
    parser = None
    subparser = None
    commands = {}

    def __init__(self, config):
        self.config = config
        self.parser = argparse.ArgumentParser(
            prog="Yoda",
            description="""Manage your repositories easier.
            Each workspaces are subcommands,
            type `yoda workspace_name -h` to show help.""")

        self.subparser = self.parser.add_subparsers(
            dest="subcommand",
            metavar="[subcommand]")

        self.__generic_arguments(self.parser)

    def add_command(self, command):
        command_name = command.__class__.__name__.lower()
        command.setup(command_name, self.config, self.subparser)
        self.commands[command_name] = command

    def parse(self):
        for key, command in self.commands.items():
            command.parse()

    def execute(self, args):
        if args.subcommand is None:
            self.parser.print_help()
            return None

        if (args.subcommand in self.commands):
            self.commands[args.subcommand].execute(args)

    def __generic_arguments(self, parser):
        parser.add_argument(
            "-d", "--debug",
            action="store_true",
            help="show debug informations")
        parser.add_argument(
            "--version",
            action="version",
            version="%(prog)s {}".format(get_version()))



# This source file is part of Yoda.
#
# Yoda is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# Yoda is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License along with
# Yoda. If not, see <http://www.gnu.org/licenses/gpl-3.0.html>.

from yoda.adapter.abstract import Abstract
from yoda.adapter.git import Git



#!/usr/bin/env python
# This source file is part of Yoda.
#
# Yoda is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# Yoda is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License along with
# Yoda. If not, see <http://www.gnu.org/licenses/gpl-3.0.html>.

import os
from setuptools.command.install import install
from setuptools import setup


class InstallCommand(install, object):
    def run(self):
        bash_completion = os.path.expanduser(
            "~/.bash_completion.d/python-argcomplete.sh"
        )
        bash_dir = os.path.dirname(bash_completion)
        if os.path.exists(bash_dir) is False:
            os.mkdir(bash_dir)

        if os.path.exists(bash_completion) is False:
            os.system(u"activate-global-python-argcomplete --dest=" + bash_dir)

        super(InstallCommand, self).run()

setup(
    name="yoda",
    version="0.1",
    description="Multiple repositories management",
    url="https://github.com/Numergy/yoda",
    license="GPLv3",
    scripts=["scripts/yoda"],
    packages=["yoda", "yoda.subcommand", "yoda.adapter"],
    install_requires=[
        u"PyYaml",
        u"pycolorizer",
        u"PrettyTable",
        u"argcomplete"
    ],
    tests_require=[
        u"mock",
        u"nose",
        u"flake8",
        u"pep8",
        u"tox",
        u"coverage"
    ],
    cmdclass={
        u"install": InstallCommand,
    }
)



# This source file is part of Yoda.
#
# Yoda is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# Yoda is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License along with
# Yoda. If not, see <http://www.gnu.org/licenses/gpl-3.0.html>.

import logging

from prettytable import PrettyTable
from pycolorizer import Color

from yoda import find_path
from yoda import Repository
from yoda import RepositoryAdapterNotFound
from yoda.subcommands import Subcommand
from yoda import Workspace


class Show(Subcommand, object):

    workspace = None
    logger = None

    def setup(self, name, config, subparser):
        self.workspace = Workspace(config)
        self.subparser = subparser
        self.logger = logging.getLogger(__name__)
        super(Show, self).setup(name, config, subparser)

    def parse(self):
        """Parse show subcommand."""
        parser = self.subparser.add_parser(
            "show",
            help="Show workspace details",
            description="Show workspace details.")

        group = parser.add_mutually_exclusive_group(required=True)
        group.add_argument('--all', action='store_true', help="All workspaces")
        group.add_argument('name', type=str, help="Workspace name", nargs='?')

    def execute(self, args):
        """Execute show subcommand."""
        if args.name is not None:
            self.show_workspace(args.name)
        elif args.all is not None:
            self.show_all()

    def show_workspace(self, name):
        """Show specific workspace."""
        if not self.workspace.exists(name):
            raise ValueError("Workspace `%s` doesn't exists." % name)

        color = Color()
        workspaces = self.workspace.list()

        self.logger.info("<== %s workspace ==>" % color.colored(name, "green"))
        self.logger.info("\tPath: %s" % workspaces[name]["path"])
        self.logger.info("\tNumber of repositories: %s"
                         % color.colored(
                             len(workspaces[name]["repositories"]),
                             "yellow"))

        repo_colored = color.colored("Repositories", "blue")
        path_colored = color.colored("Path", "blue")
        trepositories = PrettyTable(
            [repo_colored, path_colored, color.colored("+", "blue")])
        trepositories.align[repo_colored] = "l"
        trepositories.align[path_colored] = "l"

        for repo_name in workspaces[name]["repositories"]:
            fullname = "%s/%s" % (name, repo_name)
            fullpath = find_path(fullname, self.config)[fullname]
            try:
                repo = Repository(fullpath)
                repo_scm = repo.get_scm()
            except RepositoryAdapterNotFound:
                repo_scm = None
            trepositories.add_row(
                [color.colored(repo_name, "cyan"), fullpath, repo_scm])

        self.logger.info(trepositories)

    def show_all(self):
        """Show details for all workspaces."""
        for ws in self.workspace.list().keys():
            self.show_workspace(ws)
            self.logger.info("\n\n")



# This source file is part of Yoda.
#
# Yoda is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# Yoda is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License along with
# Yoda. If not, see <http://www.gnu.org/licenses/gpl-3.0.html>.

from yoda.config import Config
from yoda.logger import Logger
from yoda.repository import Repository
from yoda.repository import RepositoryAdapterNotFound
from yoda.repository import RepositoryError
from yoda.repository import RepositoryPathInvalid
from yoda.subcommands import Subcommand
from yoda.subcommands import Subcommands
from yoda.workspace import Workspace


def find_path(name, config, wsonly=False):
    """Find path for given workspace and|or repository."""
    workspace = Workspace(config)
    config = config["workspaces"]

    path_list = {}

    if name.find('/') != -1:
        wsonly = False
        ws, repo = name.split('/')
        if (workspace.exists(ws)):
            if (repo in config[ws]["repositories"]):
                path_name = "%s/%s" % (ws, repo)
                path_list[path_name] = config[ws]["repositories"][repo]

    for ws_name, ws in sorted(config.items()):
        if (name == ws_name):
            if wsonly is True:
                return {ws_name: ws["path"]}
            repositories = sorted(config[ws_name]["repositories"].items())
            for name, path in repositories:
                path_list["%s/%s" % (ws_name, name)] = path
            break

        for repo_name, repo_path in sorted(ws["repositories"].items()):
            if (repo_name == name):
                path_list["%s/%s" % (ws_name, repo_name)] = repo_path

    return path_list


def yn_choice(message, default='n'):
    choices = 'Y/n' if default.lower() in ('y', 'yes') else 'y/N'
    choice = input("%s (%s) " % (message, choices))
    values = ('y', 'yes', '') if default == 'y' else ('y', 'yes')
    return choice.strip().lower() in values



# This source file is part of Yoda.
#
# Yoda is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# Yoda is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License along with
# Yoda. If not, see <http://www.gnu.org/licenses/gpl-3.0.html>.

import logging
import os

from yoda import find_path
from yoda.subcommands import Subcommand


class Jump(Subcommand, object):

    def setup(self, name, config, subparser):
        self.subparser = subparser
        self.logger = logging.getLogger(__name__)
        super(Jump, self).setup(name, config, subparser)

    def parse(self):
        to_parser = self.subparser.add_parser(
            'jump',
            help='Jump to directory',
            description="Jump to a workspace or repository root directory.")
        to_parser.add_argument('name', type=str,
                               help='Workspace or repository name')

    def execute(self, args):
        path_list = find_path(args.name, self.config, True)

        if len(path_list) == 0:
            self.logger.error("No matches for `%s`" % args.name)
            return False

        for name, path in path_list.items():
            return self.__jump(path)

    def __jump(self, path):
        self.logger.info("Spawn new shell on `%s`" % path)
        self.logger.info(
            "Use Ctrl-D to exit and go back to the previous directory")
        os.system("cd %s; YODA_JUMP_SESSION=%s %s"
                  % (path, path, os.getenv("SHELL")))
        self.logger.info("Shell on `%s` closed." % path)



# This source file is part of Yoda.
#
# Yoda is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# Yoda is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License along with
# Yoda. If not, see <http://www.gnu.org/licenses/gpl-3.0.html>.

import logging

from pycolorizer import Color

from yoda import find_path
from yoda import Repository
from yoda.subcommands import Subcommand


class Update(Subcommand, object):

    def setup(self, name, config, subparser):
        self.subparser = subparser
        self.logger = logging.getLogger(__name__)
        super(Update, self).setup(name, config, subparser)

    def parse(self):
        """Parse update subcommand."""
        parser = self.subparser.add_parser(
            "update",
            help="Update repositories",
            description="Update one or more repositories.")

        group = parser.add_mutually_exclusive_group(required=True)
        group.add_argument('--all', action='store_true', help="All workspaces")
        group.add_argument('name', type=str, help="Repo name", nargs='?')

    def execute(self, args):
        """Execute update subcommand."""
        if args.name is not None:
            self.print_workspace(args.name)
        elif args.all is not None:
            self.print_all()

    def print_all(self):
        """Print all workspaces update."""
        for ws_name in self.config['workspaces']:
            self.print_workspace(ws_name)

    def print_workspace(self, name):
        """Print workspace update."""
        path_list = find_path(name, self.config)

        if len(path_list) == 0:
            self.logger.error("No matches for `%s`" % name)
            return False

        for name, path in path_list.items():
            self.print_update(name, path)

    def print_update(self, repo_name, repo_path):
        """Print repository update."""
        color = Color()
        try:
            repo = Repository(repo_path)
            self.logger.info(color.colored(
                "=> [%s] %s" % (repo_name, repo_path), "green"))
            repo.update()
            self.logger.info("\n")
        except ValueError as e:
            self.logger.error(e)
            pass
